# Copyright (c) 2007-2021 Andy Maleh
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

require 'glimmer/swt/widget_proxy'
require 'glimmer/swt/color_proxy'

module Glimmer
  module SWT
    # Proxy for org.eclipse.swt.custom.SashForm
    #
    # Follows the Proxy Design Pattern
    class SashFormProxy < WidgetProxy
      def initialize(*args, &block)
        super
        self.background = ColorProxy.new(230, 230, 230).swt_color
      end
    
      def post_add_content
        self.weights = @weights unless @weights.nil?
      end
      
      def set_attribute(attribute_name, *args)
        if attribute_name.to_s == 'weights'
          @weights = args
          @weights = @weights.first if @weights.first.is_a?(Array)
        else
          super(attribute_name, *args)
        end
      end
 
      def get_attribute(attribute_name)
        if attribute_name.to_s == "weights"
          swt_widget.getWeights.to_a
        else
          super(attribute_name)
        end
      end
      
    end
  end
end
